        PROCESSOR 6502

; Stay Frosty
; Last Update: October 30, 2007
; Design by:   Nathan Strum
; Program by:  Darrell Spice, Jr.

TIA_BASE_READ_ADDRESS = $30
        include VCS.H
        include macro.h
        include graphics.h
        
OVERSCAN_DELAY = $1a

HMOVE_L7          =  $70
HMOVE_L6          =  $60
HMOVE_L5          =  $50
HMOVE_L4          =  $40
HMOVE_L3          =  $30
HMOVE_L2          =  $20
HMOVE_L1          =  $10
HMOVE_0           =  $00
HMOVE_R1          =  $F0
HMOVE_R2          =  $E0
HMOVE_R3          =  $D0
HMOVE_R4          =  $C0
HMOVE_R5          =  $B0
HMOVE_R6          =  $A0
HMOVE_R7          =  $90
HMOVE_R8          =  $80


        SEG.U VARS
        ORG $80
; song player variables for main menu- note song player scratch variables
Frame             ds 1

FROSTYHEIGHT = 26
FrostyX           ds 1
FrostyY           ds 1
FrostyNoseY       ds 1
FrostyControl     ds 1 ; 76543210
                       ; 5 = 0 facing right, 1 = facing left
                       ; 43210 - 0 to 31, jump height
FrostyColorPtr    ds 2
FrostyImagePtr    ds 2
FrostyMeltX       ds 1
FrostyMeltY       ds 1

ReserveLives      ds 1

; moon or sun
CELESTIALHEIGHT = 8
CelestialX        ds 1
CelestialY        ds 1
CelestialColor    ds 1
CelestialImagePtr ds 2
SkyColor          ds 1

G48               ds 12
G48temp1          ds 1    ; holds lines remaining during Show48graphic
G48temp2          ds 1    ; holds temp image data during Show48graphic

Score             ds 3

IceColor          ds 1
PlatformColor     ds 1

Level             ds 1    ; What level are we on?
FireBall1         ds 2    ; fireball x positions in 1st section
MovingPlatform1   ds 2    ; moving platform in the 1st section
IceData1          ds 6    ; ice blocks in 1st section
FireBall2         ds 2    ; fireball x positions in 2nd section
MovingPlatform2   ds 2    ; moving platform in the 2nd section
IceData2          ds 6    ; ice blocks in 2nd section
FireBall3         ds 2    ; fireball x positions in 3rd section
MovingPlatform3   ds 2    ; moving platform in the 3rd section
IceData3          ds 6    ; ice blocks in 3rd section
FireBall4         ds 2    ; fireball x positions in 4th section
MovingPlatform4   ds 2    ; moving platform in the 4th section
IceData4          ds 6    ; ice blocks in 4th section
FireBall5         ds 2    ; fireball x positions in 5th section
MovingPlatform5   ds 2    ; moving platform in the 5th section
IceData5          ds 6    ; ice blocks in 5th section
FireBall6         ds 2    ; fireball x positions in 6th section
MovingPlatform6   ds 2    ; moving platform in the 6th section
IceData6          ds 6    ; ice blocks in 6th section

 echo "----",($00FC - *) , "bytes of RAM left"
; $FC - $FF stack
; reuse of stack RAM - no JSRs can be called when these are used

; end reuse of stack RAM

        SEG CODE


        ORG $F000
InitSystem
        cmp SelectLastBank   ; inits system then goes to the menu
        jmp InitSystemCode   ; jump to Main Menu

InitSystemCode
InitFrostyCode    ; starting point of this game
        CLEAN_START
        lda #$7b
        sta CelestialX
        lda #$11
        sta CelestialY
        lda #$18 ; NTSC Yellow
        sta CelestialColor
        lda #<SunGraphic
        sta CelestialImagePtr
        lda #>SunGraphic
        sta CelestialImagePtr+1
        
        lda #0
        sta Score
        sta Score+1
        sta Score+2
        
        lda #$74 ; NTSC Purple
        sta PlatformColor
        
        lda #50
        sta FrostyX
        sta FrostyY
        
        jmp VerticalBlankCode
        
PosObject   ; A holds X value
        sec ; X holds object, 0=P0, 1=P1, 2=M0, 3=M1, 4=Ball
        sta WSYNC
DivideLoop
        sbc #15
        bcs DivideLoop ;+4    4
        eor #7
        asl
        asl
        asl
        asl            ;+10  14
        sta.wx HMP0,X  ;+5   19
        sta RESP0,X    ;+4   23
        sta WSYNC
        sta HMOVE
        rts        
        
        
VerticalBlankCode
        lda #$02
        sta WSYNC
        sta VSYNC  ; 3   start vertical sync, D1=1
        sta VBLANK ; 3 6 start vertical blank, D1=1
        lda #$2C   ; 2 8 set timer for end of Vertical Blank
        sta TIM64T
        sta WSYNC  ; 1st line of vertical sync
        inc Frame  ; 5
        sta WSYNC  ; 2nd line of vertical sync
        lda #0     ; 2
        sta WSYNC  ; 3rd line of vertical sync
        sta VSYNC  ; stop vertical sync, D1 = 0
        
        ; game code here
        
        lda #<FrostyGraphic
        Sta FrostyImagePtr
        lda #>FrostyGraphic
        sta FrostyImagePtr+1

        lda #<FrostyColors
        sta FrostyColorPtr
        lda #>FrostyColors
        sta FrostyColorPtr+1        
        
        ;test sun in various X positions
        lda Frame
        lsr
        clc        
        adc #20
        sta CelestialX
        ; end test sun

        lda CelestialX
        ldx #0 ; sun/moon in Sprite 0
        jsr PosObject
        lda CelestialColor
        sta COLUP0
        lda #15
        sta COLUPF
        lda #$86 ; NTSC Blue
        sta COLUBK ;
        lda #1
        sta CTRLPF ; reflected playfield for HORIZON
        lda #0
        sta HMP0
        
        ; test ReserveLives display
        lda Frame
        eor #$FF
        lsr
        lsr
        lsr
        lsr
        lsr
        and #%11
        sta ReserveLives
        
        ; test score
        clc
        sed
        lda #$01
        adc Score+2
        sta Score+2
        lda #0
        adc Score+1
        sta Score+1
        lda #0
        adc Score
        sta Score
        cld
        
        ; position nose
        sec
        lda FrostyY
        sbc #10
        sta FrostyNoseY
        lda FrostyControl
        and #%00100000
        beq .noseOnRight
        lda #2
        .byte $2c
.noseOnRight
        lda #4
        clc
        adc FrostyX
        ldx #3
        jsr PosObject
        lda #$20
        sta NUSIZ1           ; 3  25
        ldx #$38 ; NTSC Orange for nose
        stx COLUP1
        lda #0
        sta HMM1        
        
        lda FrostyX
        cmp CelestialX
        bcc .facingLeft
        lda #%00001000
        sta REFP0
.facingLeft        

        jmp KernelCode
        
.skipCelestialDraw
        lda #0
        beq .celestialDraw
        
KernelCode

.VblankWait
        lda INTIM
        bpl .VblankWait
        lda #0
        sta VBLANK ; turn on video output
        sta WSYNC
        
        ldx #16
.HorizonLoop        
        sta WSYNC
        cpx #8                        ; 2
        bcs .skipHills                ; 2  4
        lda HillColor,x
        sta COLUPF
        lda Hill0,x                   ; 4  8
        sta PF0                       ; 3 11
        lda Hill1,x                   ; 4 15
        sta PF1                       ; 3 18
        lda Hill2,x                   ; 4 22
        sta PF2                       ; 3 25
.skipHills     
        txa
        sec
        sbc CelestialY
        adc #CELESTIALHEIGHT
        bcc .skipCelestialDraw
        tay
        lda (CelestialImagePtr),y     ; 
.celestialDraw        
        sta GRP0                      ;
        dex                           ; 2 44
        bpl .HorizonLoop              ; 2 46
        inx
        stx WSYNC
        stx CTRLPF ; repeated playfield for game screen
        stx COLUBK
        stx PF0
        stx PF1
        stx PF2
        lda FrostyControl
        lsr
        lsr
        sta REFP0   ; set Frosty facing left or right
        lda FrostyX
        ldx #0
        jsr PosObject
        
        lda #0
        ldx #148
.testloop        
        sta WSYNC
        sta GRP0
        sty COLUP0
        cpx FrostyNoseY
        php
        pla
        sta ENAM1
        txa
        sec
        sbc FrostyY
        adc #FROSTYHEIGHT
        bcc .skipDrawTL
        tay
        lda (FrostyImagePtr),y
.frostyDraw        
        pha
        lda (FrostyColorPtr),y
        tay
        pla
        dex
        bne .testloop
        beq .score
.skipDrawTL      
        lda #0
        beq .frostyDraw
        
.score        

        

        
        
; display SCORE and Reserve Lives
        sta WSYNC
        lda #0               ; 2
        sta GRP0             ; 3   5
        SLEEP 3              ; 3   8
        sta PF0              ; 3  11
        sta PF0              ; 3  14
        sta PF0              ; 3  17
        lda #3               ; 2  19
        sta NUSIZ0           ; 3  22
        sta NUSIZ1           ; 3  25
        sta VDELP0           ; 3  28
        sta VDELP1           ; 3  31
        lda #HMOVE_R1             ; 2  33
        sta HMP0             ; 3  36 x adjust for P0  
        sta RESP0            ; 3  39 set x for P0
        sta RESP1            ; 3  42 set x for P1
        sta WSYNC
        SLEEP 38             ; 38
        sta RESM1            ;  3  41 set x for M1 - used to draw nose in lives remaining
        sta WSYNC
        sta HMOVE   ; fine adjust for P0
        lda #15 ; WHITE NTSC & PAL
        sta COLUP0
        sta COLUP1
        lda #0
        sta HMP0        
        sta REFP0       ; reset in case Frosty was facing left
        
; Load G48 with SCORE graphics        
        lda #>ZeroGraphic
        sta G48+1
        sta G48+3
        sta G48+5
        sta G48+7
        sta G48+9
        sta G48+11
        
        lda Score
        and #$F0
        lsr
        sta G48
        lda Score
        and #$0f
        asl
        asl
        asl
        sta G48+2
        
        lda Score+1
        and #$F0
        lsr
        sta G48+4
        lda Score+1
        and #$0F
        asl
        asl
        asl
        sta G48+6
        
        lda Score+2
        and #$F0
        lsr
        sta G48+8
        lda Score+2
        and #$0F
        asl
        asl
        asl
        sta G48+10
        
        sta WSYNC                 
        ldx #0
BlankZeroLoop        
        lda G48,x
        bne BZLdone
        lda #<BlankGraphic
        sta G48,x
        inx
        inx
        cpx #10
        bcc BlankZeroLoop
BZLdone    
        SLEEP 10
        inx
        inx
        cpx #10
        bcc BZLdone

        ; show score
        ldy #7        
Show48graphic 
        ; call with Y holding the lines-1 to show
        ; G48 thru G48+$B must be preloaded with pointers to the
        ; 48 pixel graphic image
        STY G48temp1
        sta WSYNC
.graphicLoop
        ldy G48temp1         ;+3  63  189
        lda (G48),y          ;+5  68  204
        sta GRP0             ;+3  71  213      D1     --      --     --
        sta WSYNC            ;go
        lda (G48+$2),y       ;+5   5   15
        sta GRP1             ;+3   8   24      D1     D1      D2     --
        lda (G48+$4),y       ;+5  13   39
        sta GRP0             ;+3  16   48      D3     D1      D2     D2
        lda (G48+$6),y       ;+5  21   63
        sta G48temp2         ;+3  24   72
        lda (G48+$8),y       ;+5  29   87
        tax                  ;+2  31   93
        lda (G48+$A),y       ;+5  36  108
        tay                  ;+2  38  114
        lda G48temp2         ;+3  41  123              !
        sta GRP1             ;+3  44  132      D3     D3      D4     D2!
        stx GRP0             ;+3  47  141      D5     D3!     D4     D4
        sty GRP1             ;+3  50  150      D5     D5      D6     D4!
        sta GRP0             ;+3  53  159      D4*    D5!     D6     D6
        dec G48temp1         ;+5  58  174                             !
        bpl .graphicLoop     ;+2  60  180
        lda #0
        sta GRP0
        sta GRP1
        sta GRP0
        sta GRP1
        sta WSYNC
        
        sta WSYNC
        ldx #0 ; black for nose when no lives remaining        
        lda ReserveLives
        beq Only2left  ; doesn't matter - graphic will be blank
        ldx #$38 ; NTSC Orange for nose
        cmp #2
        bcc Only1left
        beq Only2left
        lda #$23       ; show 3 close
        .byte $2c
Only2left        
        lda #$21       ; show 2 close
        .byte $2c
Only1left        
        lda #$20       ; show 1
        sta NUSIZ0
        stx COLUP1
        
        sta NUSIZ1
        lda #0
        sta VDELP0           ; 3  28
        sta VDELP1           ; 3  31          
        
        lda ReserveLives
        bne AtLeast1left
        lda #<BlankGraphic
        .byte $2c ; sucks up the next command
AtLeast1left        
        lda #<LivesGraphic
        sta G48
        lda #>LivesGraphic
        sta G48+1

        ldx #ENAM1
        txs
        ldy #12
.graphicLoop2
        lda LivesColor,y
        sta WSYNC
        sta COLUP0
        lda (G48),y
        sta GRP0
        cpy #3 ; enable carrot nose on line 3
        php
        pla
        dey
        bpl .graphicLoop2
        lda #0
        sta WSYNC
        sta GRP0
        sta NUSIZ0           ; 3  22
        ldx #$FF
        txs        

        
        
OverScanCode
        sta WSYNC
        lda #2
        sta VBLANK ; turn off video output
        lda #OVERSCAN_DELAY
        sta TIM64T
        
        lda SWCHA
        bmi SkipRight
        inc FrostyX
        tay
        lda FrostyControl
        and #%11011111     ; set facing right 
        sta FrostyControl
        tya
SkipRight
        rol
        bmi SkipLeft
        dec FrostyX
        tay
        lda FrostyControl
        ora #%00100000     ; set facing left
        sta FrostyControl
        tya
SkipLeft
        rol
        bmi SkipDown
        dec FrostyY
SkipDown
        rol
        bmi SkipUp
        inc FrostyY
SkipUp     
        lda FrostyX
        cmp #160
        bne .nextXcheck
        lda #0
        beq .saveNewX
.nextXcheck
        cmp #255
        bne .noXadjust
        lda #159
.saveNewX        
        sta FrostyX
.noXadjust        
        lda FrostyY
        cmp #$1b
        bne .nextYcheck
        lda #$1c
        bne .saveNewY
.nextYcheck
        cmp #$96
        bne .noYadjust
        lda #$95
.saveNewY
        sta FrostyY
.noYadjust        
        
        
 
        
.OscanWait
        lda SWCHB
        eor #$FF
        and #%11
        bne SelectOrReset
        lda INTIM
        bpl .OscanWait
        jmp VerticalBlankCode
        
SelectOrReset
        jmp InitFrostyCode
        
        
SunGraphic
        .byte zz___XX___
        .byte zz__X__X__
        .byte zz_X_XX_X_
        .byte zz_XXXXXX_
        .byte zz_X_X_XX_
        .byte zz_X_X_XX_
        .byte zz__XXXX__
        .byte zz___XX___
        
HillColor
        .byte 4,4,8,8,15,15,15,15
        
        
Hill0
        .byte zzXXXX____ ; 0
        .byte zzXXXX____ ; 1
        .byte zzXXXX____ ; 2
        .byte zzXXXX____ ; 3
        .byte zzXXXX____ ; 4
        .byte zzXXX_____ ; 5
        .byte zzXX______ ; 6
        .byte zzX_______ ; 7
        
Hill1
        .byte zzXXXXXXXX ; 0
        .byte zzXXXXXXXX ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXX_ ; 4
        .byte zzXXXXXXX_ ; 5
        .byte zzX_XXXX__ ; 6
        .byte zz___XX___ ; 7
        
        
        
Hill2   
        .byte zzXXXXXXXX ; 0
        .byte zz_XXXXXXX ; 1
        .byte zz__XXXXXX ; 2
        .byte zz___XXXX_ ; 3
        .byte zz____XX__ ; 4
        .byte zz________ ; 5
        .byte zz________ ; 6
        .byte zz________ ; 7
        
        align 256
        
Level1
        .byte 40 ; section height
        .byte 0,0,0,zz____XX_X, zzX_XX____,0,0 ; initial ice blocks
        .byte 0,0,0,zzXXXX____, zzXXXXXX__, 0
        .byte 40 ; section height
        .byte 0,0,0,zz____XX_X, zzX_XX____,0,0 ; initial ice blocks
        .byte 0,0,0,zzXXXX____, zzXXXXXX__, 0
        .byte 40 ; section height
        .byte 0,0,0,zz____XX_X, zzX_XX____,0,0 ; initial ice blocks
        .byte 0,0,0,zzXXXX____, zzXXXXXX__, 0
        .byte 40 ; section height
        .byte 0,0,0,zz____XX_X, zzX_XX____,0,0 ; initial ice blocks
        .byte 0,0,0,zzXXXX____, zzXXXXXX__, 0
        
        align 256
ZeroGraphic  
        .byte zz___XX___ ; 0
        .byte zz__X__X__ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_X____X_ ; 3
        .byte zz_X____X_ ; 4
        .byte zz_X____X_ ; 5
        .byte zz__X__X__ ; 6
        .byte zz___XX___ ; 7

OneGraphic
        .byte zz__XXXXX_ ; 0
        .byte zz____X___ ; 1
        .byte zz____X___ ; 2
        .byte zz____X___ ; 3
        .byte zz____X___ ; 4
        .byte zz__X_X___ ; 5
        .byte zz___XX___ ; 6
        .byte zz____X___ ; 7
        
TwoGraphic
        .byte zz_XXXXXX_ ; 0
        .byte zz_X______ ; 1
        .byte zz_X______ ; 2
        .byte zz__X_____ ; 3
        .byte zz___XX___ ; 4
        .byte zz_____X__ ; 5
        .byte zz_X____X_ ; 6
        .byte zz__XXXX__ ; 7
        
ThreeGraphic
        .byte zzXXXXX___ ; 0
        .byte zz___XXX__ ; 1
        .byte zz____XX__ ; 2
        .byte zz___XXX__ ; 3
        .byte zz__XXX___ ; 4
        .byte zz____XX__ ; 5
        .byte zz_XX__XX_ ; 6
        .byte zz__XXXX__ ; 7
        
FourGraphic
        .byte zz____X___ ; 0
        .byte zz____X___ ; 1
        .byte zz____X___ ; 2
        .byte zz____X___ ; 3
        .byte zz_XXXXXX_ ; 4
        .byte zz__X_X___ ; 5
        .byte zz___XX___ ; 6
        .byte zz____X___ ; 7
        
FiveGraphic
        .byte zzXXXXX___ ; 0
        .byte zz____XX__ ; 1
        .byte zz_____X__ ; 2
        .byte zz____XX__ ; 3
        .byte zz_XXXXX__ ; 4
        .byte zz_XXX____ ; 5
        .byte zz_XX_____ ; 6
        .byte zz__XXXXX_ ; 7

SixGraphic
        .byte zz___XX___ ; 0
        .byte zz__X__X__ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_XX__X__ ; 3
        .byte zz_X_XX___ ; 4
        .byte zz_X______ ; 5
        .byte zz__X__X__ ; 6
        .byte zz___XX___ ; 7
        
SevenGraphic
        .byte zzXX______ ; 0
        .byte zz_XX_____ ; 1
        .byte zz__XX____ ; 2
        .byte zz__XX____ ; 3
        .byte zz___XX___ ; 4
        .byte zz___XX___ ; 5
        .byte zz____XX__ ; 6
        .byte zz_XXXXXX_ ; 7
        
EightGraphic
        .byte zz___XX___ ; 0
        .byte zz__X__X__ ; 1
        .byte zz_X____X_ ; 2
        .byte zz_X____X_ ; 3
        .byte zz__XXXX__ ; 4
        .byte zz_X____X_ ; 5
        .byte zz__X__X__ ; 6
        .byte zz___XX___ ; 7
        
NineGraphic
        .byte zz___XX___ ; 0
        .byte zz__X__X__ ; 1
        .byte zz______X_ ; 2
        .byte zz___XX_X_ ; 3
        .byte zz__X__XX_ ; 4
        .byte zz_X____X_ ; 5
        .byte zz__X__X__ ; 6
        .byte zz___XX___ ; 7
        
BlankGraphic
        .byte zz________ ; 0
        .byte zz________ ; 1
        .byte zz________ ; 2
        .byte zz________ ; 3
        .byte zz________ ; 4
        .byte zz________ ; 5
        .byte zz________ ; 6
        .byte zz________ ; 7
        .byte zz________ ; 8
        .byte zz________ ; 9
        .byte zz________ ; 10
        .byte zz________ ; 11
        .byte zz________ ; 12
        
LivesGraphic
        .byte zz___XX___ ; 0
        .byte zz__X__X__ ; 1
        .byte zz_X_XX_X_ ; 2
        .byte zz_XXX____ ; 3  ; gap for nose drawn by missile 1
        .byte zz_XX_X_X_ ; 4
        .byte zz_XX_X_X_ ; 5
        .byte zz__XXXX__ ; 6
        .byte zz___XX___ ; 7
        .byte zz__XXXX__ ; 8
        .byte zz___XX___ ; 9
        .byte zz___XX___ ; 10
        .byte zz___XX___ ; 11
        .byte zz___XX___ ; 12

LivesColor
        .byte 15,15,15,15,15,15,15,15,4,$42,4,4,6
        
FrostyGraphic
        .byte zz__XXXX__ ; 0
        .byte zz_XXXXXX_ ; 1
        .byte zzXXXXXXXX ; 2
        .byte zzXXXXXXXX ; 3
        .byte zzXXXXXXXX ; 4
        .byte zzXXXXXXXX ; 5
        .byte zzXXXX_XXX ; 6
        .byte zz_XXXXXX_ ; 7
        .byte zz__XXXX__ ; 8
        .byte zz_XXXXXX_ ; 9
        .byte zz_XXX_XX_ ; 10
        .byte zz_XXXXXX_ ; 11
        .byte zz_XXXXXX_ ; 12
        .byte zz_XXX_XX_ ; 13
        .byte zz__XXXX__ ; 14
        .byte zz___XX___ ; 15
        .byte zz__X__X__ ; 16
        .byte zz__XX____ ; 17
        .byte zz__X_X___ ; 18
        .byte zz__XXXX__ ; 19
        .byte zz___XX___ ; 20
        .byte zz__XXXX__ ; 21
        .byte zz___XX___ ; 22
        .byte zz___XX___ ; 23
        .byte zz___XX___ ; 24
        .byte zz___XX___ ; 25
        
FrostyColors    
        .byte 8   ; 0
        .byte 10  ; 1
        .byte 12  ; 2
        .byte 12  ; 3
        .byte 14  ; 4
        .byte 14  ; 5
        .byte 14  ; 6
        .byte 12  ; 7
        .byte 8   ; 8
        .byte 10  ; 9
        .byte 12  ; 10
        .byte 14  ; 11
        .byte 14  ; 12
        .byte 14  ; 13
        .byte 12  ; 14
        .byte 10  ; 15
        .byte 12  ; 16
        .byte 14  ; 17
        .byte 14  ; 18
        .byte 14  ; 19
        .byte 10  ; 20
        .byte 4   ; 21
        .byte $42 ; 22 NTSC red
        .byte 4   ; 23
        .byte 4   ; 24
        .byte 6   ; 25

        echo "----",($FFF4 - *) , "bytes of ROM left"
        ORG $FFF4       ; assuming 16K or 32K
SelectBank1 .byte $00
SelectBank2 .byte $00
SelectBank3 .byte $00
SelectBank4 .byte $00
SelectBank5 .byte $00
SelectBank6 .byte $00  ; SelectLastBank16K
SelectBank7 .byte $00
SelectLastBank .byte $00
;       .word InitSystem ; NMI and 8 overlap NMI
        .word InitSystem ; RESET
        .word InitSystem ; IRQ
